@@ -68,15 +68,15 @@ from requests.adapters import HTTPAdapter
 # ujson is faster than Python's builtin json module
 import ujson
 
-from .utils import skeleton_df_to_nx, heal_skeleton, skeleton_df_to_swc
-
 logger = logging.getLogger(__name__)
 DEFAULT_NEUPRINT_CLIENT = None
 NEUPRINT_CLIENTS = {}
 
+
 class NeuprintTimeoutError(HTTPError):
     pass
 
+
 def setup_debug_logging():
     """
     Simple debug logging configuration.
@@ -228,7 +228,7 @@ def verbose_errors(f):
             # If the error response body is non-empty, show that in the traceback, too.
             # neuprint-http error messages are often helpful -- show them!
             if hasattr(ex, 'response_content_appended') or (ex.response is None and ex.request is None):
-                raise # Nothing to add to the exception.
+                raise  # Nothing to add to the exception.
 
             msg = ""
 
@@ -363,7 +363,6 @@ class Client:
         self.primary_rois = sorted(self.meta['primaryRois'])
         self.all_rois = _all_rois_from_meta(self.meta)
 
-
     def __repr__(self):
         s = f'Client("{self.server}", "{self.dataset}"'
         if not self.verify:
@@ -381,20 +380,18 @@ class Client:
         r.raise_for_status()
         return r
 
-
     def _fetch_raw(self, url, json=None, ispost=False):
         return self._fetch(url, json=json, ispost=ispost).content
 
-
     def _fetch_json(self, url, json=None, ispost=False):
         r = self._fetch(url, json=json, ispost=ispost)
         return ujson.loads(r.content)
 
-
     ##
     ## CUSTOM QUERIES
     ##
     ## Note: Transaction queries are not implemented here.  See admin.py
+    ##
 
     def fetch_custom(self, cypher, dataset="", format='pandas'):
         """
@@ -421,7 +418,6 @@ class Client:
         url = f"{self.server}/api/custom/custom"
         return self._fetch_cypher(url, cypher, dataset, format)
 
-
     def _fetch_cypher(self, url, cypher, dataset, format='pandas'):
         """
         Fetch cypher from an endpoint.
@@ -433,8 +429,7 @@ class Client:
             msg = ("Your cypher query contains 'smart quotes' (e.g. ‘foo’ or “foo”),"
                    " which are not valid characters in cypher."
                    " Please replace them with ordinary quotes (e.g. 'foo' or \"foo\").\n"
-                   "Your query was:\n"
-                   + cypher)
+                   "Your query was:\n" + cypher)
             raise RuntimeError(msg)
 
         dataset = dataset or self.dataset
@@ -452,7 +447,6 @@ class Client:
         df = pd.DataFrame(result['data'], columns=result['columns'])
         return df
 
-
     ##
     ## API-META
     ##
@@ -463,28 +457,24 @@ class Client:
         """
         return self._fetch_json(f"{self.server}/api/available")
 
-
     def fetch_help(self):
         """
         Fetch auto-generated REST API documentation, as YAML text.
         """
         return self._fetch_raw(f"{self.server}/api/help/swagger.yaml").decode('utf-8')
 
-
     def fetch_server_info(self):
         """
         Returns whether or not the server is public.
         """
         return self._fetch_json(f"{self.server}/api/serverinfo")['IsPublic']
 
-
     def fetch_version(self):
         """
         Returns the version of the ``neuPrintHTTP`` server.
         """
         return self._fetch_json(f"{self.server}/api/version")['Version']
 
-
     ##
     ## DB-META
     ##
@@ -495,28 +485,24 @@ class Client:
         """
         return self._fetch_json(f"{self.server}/api/dbmeta/database")
 
-
     def fetch_datasets(self):
         """
         Fetch basic information about the available datasets on the server.
         """
         return self._fetch_json(f"{self.server}/api/dbmeta/datasets")
 
-
     def fetch_instances(self):
         """
         Fetch secondary data instances avaiable through neupint http
         """
         return self._fetch_json(f"{self.server}/api/dbmeta/instances")
 
-
     def fetch_db_version(self):
         """
         Fetch the database version
         """
         return self._fetch_json(f"{self.server}/api/dbmeta/version")['Version']
 
-
     ##
     ## USER
     ##
@@ -528,7 +514,6 @@ class Client:
         """
         return self._fetch_json(f"{self.server}/profile")
 
-
     def fetch_token(self):
         """
         Fetch your user authentication token.
@@ -541,7 +526,6 @@ class Client:
         """
         return self._fetch_json(f"{self.server}/token")['token']
 
-
     ##
     ## Cached
     ##
@@ -574,7 +558,6 @@ class Client:
 
         return result['info'], conn_df, skel_df
 
-
     def fetch_roi_completeness(self, format='pandas'):
         """
         Fetch the pre-computed traced "completeness" statistics
@@ -596,7 +579,6 @@ class Client:
         df = pd.DataFrame(result['data'], columns=result['columns'])
         return df
 
-
     def fetch_roi_connectivity(self, format='pandas'):
         """
         Fetch the pre-computed connectivity statistics
@@ -628,7 +610,6 @@ class Client:
         df = pd.DataFrame(weights, columns=['from_roi', 'to_roi', 'count', 'weight'])
         return df
 
-
     ##
     ## ROI MESHES
     ##
@@ -658,11 +639,10 @@ class Client:
                 f.write(data)
         return data
 
-
     ##
     ## SKELETONS
     ##
-    def fetch_skeleton(self, body, heal=False, export_path=None, format='pandas'):
+    def fetch_skeleton(self, body, heal=False, export_path=None, format='pandas', with_distances=False):
         """
         Fetch the skeleton for a neuron or segment.
 
@@ -678,6 +658,10 @@ class Client:
                 via their pairwise nearest neighbors. See :py:func:`.heal_skeleton()`
                 for more details.
 
+                If you want the healing procedure to refrain from connecting very
+                distant fragments, set ``heal`` to a maximum allowed distance,
+                e.g. ``heal=1000.0``
+
             format (str):
                 Either 'pandas', 'swc' (similar to CSV), or 'nx' (``networkx.DiGraph``).
 
@@ -685,6 +669,15 @@ class Client:
                 Optional. Writes the ``.swc`` file to disk.
                 (SWC format is written, regardless of the returned ``format``.)
 
+            with_distances:
+                Only valid when format is ``pandas`` or ``nx``.
+                If True, a 'distance' column (or edge attribute) will be added
+                to the dataframe (or nx.Graph), indicating the distances from each
+                node to its parent node.
+                In DataFrame results, root nodes will be assigned a distance of ``np.inf``.
+                Distances are computed AFTER healing is performed.
+                Distances will not be present in any exported SWC file.
+
         Returns:
 
             Either a string (swc), a DataFrame (pandas), or ``networkx.DiGraph`` (nx).
@@ -695,27 +688,25 @@ class Client:
             - :py:func:`.skeleton_df_to_nx()`
             - :py:func:`.skeleton_df_to_swc()`
         """
+        from .skeleton import skeleton_df_to_nx, heal_skeleton, skeleton_df_to_swc, skeleton_swc_to_df, calc_segment_distances
+
         try:
             body = int(body)
         except ValueError:
             raise RuntimeError(f"Please pass an integer body ID, not '{body}'")
 
-        assert isinstance(heal, bool), "Bad argument: 'heal' should be a bool."
         assert format in ('swc', 'pandas', 'nx'), f'Invalid format: {format}'
+        assert not with_distances or format in ('pandas', 'nx'), \
+            f"The with_distances option can only be used with the 'pandas' or 'nx' output formats, not {format}"
 
         url = f"{self.server}/api/skeletons/skeleton/{self.dataset}/{body}?format=swc"
         swc = self._fetch_raw(url, ispost=False).decode('utf-8')
 
         if heal or format != 'swc':
-            cols = ['rowId', 'node_type', 'x', 'y', 'z', 'radius', 'link']
-            lines = swc.split('\n')
-            lines = filter(lambda line: '#' not in line, lines)
-            swc_csv = '\n'.join(lines)
-            df = pd.read_csv(StringIO(swc_csv), delimiter=' ', engine='c', names=cols, header=None)
-            df = df.drop(columns=['node_type'])
+            df = skeleton_swc_to_df(swc)
 
         if heal:
-            df = heal_skeleton(df)
+            df = heal_skeleton(df, heal)
             if export_path or format == 'swc':
                 swc = skeleton_df_to_swc(df)
 
@@ -727,14 +718,15 @@ class Client:
             return swc
 
         if format == 'pandas':
+            if with_distances:
+                df['distance'] = calc_segment_distances(df)
             return df
 
         if format == 'nx':
-            return skeleton_df_to_nx(df)
+            return skeleton_df_to_nx(df, with_distances=with_distances)
 
         raise AssertionError('Should not get here.')
 
-
     ##
     ## RAW KEY-VALUE
     ##
@@ -747,7 +739,6 @@ class Client:
         url = f"{self.server}/api/raw/keyvalue/key/{instance}/{key}"
         return self._fetch_raw(url, ispost=False)
 
-
     def post_raw_keyvalue(self, instance, key, value):
         """
         Post a value from the ``neuprintHTTP`` server.
