@@ -13,6 +13,7 @@ import pandas as pd
 from .utils import make_args_iterable
 from .client import inject_client
 
+
 def neuroncriteria_args(*argnames):
     """
     Returns a decorator.
@@ -50,7 +51,7 @@ def copy_as_neuroncriteria(obj, client=None):
         None -> NC()
 
         int -> NC(bodyId)
-        [str,...] -> NC(bodyId)
+        [int,...] -> NC(bodyId)
         DataFrame['bodyId'] -> NC(bodyId)
 
         str -> NC(type, instance)
@@ -134,6 +135,7 @@ class NeuronCriteria:
                   min_pre=0, min_post=0,
                   rois=None, inputRois=None, outputRois=None, min_roi_inputs=1, min_roi_outputs=1,
                   label=None, roi_req='all',
+                  soma=None,
                   client=None ):
         """
         Except for ``matchvar``, all parameters must be passed as keyword arguments.
@@ -222,10 +224,6 @@ class NeuronCriteria:
                 How many output (pre) synapses a neuron must have in each ROI to satisfy the
                 ``outputRois`` criteria.   Can only be used if you provided ``outputRois``.
 
-            roi_req (Either ``'any'`` or ``'all'``):
-                Whether a neuron must intersect all of the listed input/output ROIs, or any of the listed input/output ROIs.
-                When using 'any', each neuron must still match at least one input AND at least one output ROI.
-
             label (Either ``'Neuron'`` or ``'Segment'``):
                 Which node label to match with.
                 (In neuprint, all ``Neuron`` nodes are also ``Segment`` nodes.)
@@ -233,6 +231,14 @@ class NeuronCriteria:
                 In that case, ``'Segment'`` is the default. (It's assumed you're really interested
                 in the bodies you explicitly listed, whether or not they have the ``'Neuron'`` label.)
 
+            roi_req (Either ``'any'`` or ``'all'``):
+                Whether a neuron must intersect all of the listed input/output ROIs, or any of the listed input/output ROIs.
+                When using 'any', each neuron must still match at least one input AND at least one output ROI.
+
+            soma (Either ``True``, ``False``, or ``None``)
+                If ``True``, only return neurons with a ``somaLocation``.
+                If ``False``, return neurons eithout a ``somaLocation``.
+
             client (:py:class:`neuprint.client.Client`):
                 Used to validate ROI names.
                 If not provided, the global default ``Client`` will be used.
@@ -260,18 +266,17 @@ class NeuronCriteria:
                 label = 'Segment'
         assert label in ('Neuron', 'Segment'), f"Invalid label: {label}"
 
-        assert not regex or len(instance) <= 1, \
-            "Please provide only one regex pattern for instance"
-        assert not regex or len(type) <= 1, \
-            "Please provide only one regex pattern for type"
+        if not regex and instance:
+            for i in instance:
+                assert isinstance(i, (str, None)), 'instance should be a string, or None'
+                assert instance is None or '.*' not in i, \
+                    f"instance appears to be a regular expression ('{i}'), but you didn't pass regex=True"
 
-        if not regex and instance and len(instance) == 1 and isinstance(instance[0], str):
-            assert '.*' not in instance[0], \
-                f"instance appears to be a regular expression ('{instance[0]}'), but you didn't pass regex=True"
-
-        if not regex and type and len(type) == 1 and isinstance(type[0], str):
-            assert '.*' not in type[0], \
-                f"type appears to be a regular expression ('{type[0]}'), but you didn't pass regex=True"
+        if not regex and type:
+            for t in type:
+                assert isinstance(t, (str, None)), 'type should be a string, or None'
+                assert type is None or '.*' not in t, \
+                    f"type appears to be a regular expression ('{t}'), but you didn't pass regex=True"
 
         assert roi_req in ('any', 'all')
 
@@ -280,6 +285,9 @@ class NeuronCriteria:
         assert min_roi_outputs <= 1 or outputRois, \
             "Can't stipulate min_roi_outputs without a list of outputRois"
 
+        assert soma in (True, False, None), \
+            f"soma must be True, False, or None, not {soma}"
+
         # If the user provided both intersecting rois and input/output rois,
         # force them to make the intersecting set a superset of the others.
         rois = {*rois}
@@ -322,6 +330,7 @@ class NeuronCriteria:
         self.regex = regex
         self.label = label
         self.roi_req = roi_req
+        self.soma = soma
 
 
     def __eq__(self, value):
@@ -342,7 +351,7 @@ class NeuronCriteria:
                  'bodyId', 'instance', 'type', 'status',
                  'cropped', 'min_pre', 'min_post', 'rois', 'inputRois',
                  'outputRois', 'min_roi_inputs', 'min_roi_outputs',
-                 'regex', 'label', 'roi_req']
+                 'regex', 'label', 'roi_req', 'soma']
 
         for at in params:
             me = getattr(self, at)
@@ -422,6 +431,9 @@ class NeuronCriteria:
         if self.roi_req != 'all':
             s += f', roi_req="{self.roi_req}"'
 
+        if self.soma is not None:
+            s += f', soma="{self.soma}"'
+
         s += ')'
 
         return s
@@ -441,12 +453,13 @@ class NeuronCriteria:
             var = f"{self.matchvar}_search_bodyIds"
             exprs[var] = (f"{[*values]} as {var}")
 
-        if len(self.type) > self.MAX_LITERAL_LENGTH:
+        # Never store regular expressions into a global
+        if not self.regex and len(self.type) > self.MAX_LITERAL_LENGTH:
             values = [*filter(lambda s: s is not None, self.type)]
             var = f"{self.matchvar}_search_types"
             exprs[var] = f"{[*values]} as {var}"
 
-        if len(self.instance) > self.MAX_LITERAL_LENGTH:
+        if not self.regex and len(self.instance) > self.MAX_LITERAL_LENGTH:
             values = [*filter(lambda s: s is not None, self.instance)]
             var = f"{self.matchvar}_search_instances"
             exprs[var] = f"{[*values]} as {var}"
@@ -487,7 +500,8 @@ class NeuronCriteria:
         the WHERE clause of a cypher query.
         """
         exprs = [self.bodyId_expr(), self.typeinst_expr(), self.cbf_expr(), self.status_expr(),
-                 self.cropped_expr(), self.rois_expr(), self.pre_expr(), self.post_expr()]
+                 self.cropped_expr(), self.rois_expr(), self.pre_expr(), self.post_expr(),
+                 self.soma_expr()]
         exprs = [*filter(None, exprs)]
         return exprs
 
@@ -516,13 +530,15 @@ class NeuronCriteria:
 
     def instance_expr(self):
         valuevar = None
-        if len(self.instance) > self.MAX_LITERAL_LENGTH:
+        # We never use a global 'valuevar' in the regex case
+        if not self.regex and len(self.instance) > self.MAX_LITERAL_LENGTH:
             valuevar = f"{self.matchvar}_search_instances"
         return where_expr('instance', self.instance, self.regex, self.matchvar, valuevar)
 
     def type_expr(self):
         valuevar = None
-        if len(self.type) > self.MAX_LITERAL_LENGTH:
+        # We never use a global 'valuevar' in the regex case
+        if not self.regex and len(self.type) > self.MAX_LITERAL_LENGTH:
             valuevar = f"{self.matchvar}_search_types"
         return where_expr('type', self.type, self.regex, self.matchvar, valuevar)
 
@@ -558,7 +574,6 @@ class NeuronCriteria:
         roi_logic = {'any': 'OR', 'all': 'AND'}[self.roi_req]
         return "(" + f" {roi_logic} ".join(f"{self.matchvar}.`{roi}`" for roi in rois) + ")"
 
-
     def pre_expr(self):
         if self.min_pre:
             return f"{self.matchvar}.pre >= {self.min_pre}"
@@ -571,6 +586,13 @@ class NeuronCriteria:
         else:
             return ""
 
+    def soma_expr(self):
+        if self.soma is None:
+            return ""
+        if self.soma:
+            return f"{self.matchvar}.somaLocation IS NOT NULL"
+        else:
+            return f"{self.matchvar}.somaLocation IS NULL"
 
     def all_conditions(self, *vars, prefix=0, comments=True):
         if isinstance(prefix, int):
@@ -593,7 +615,6 @@ class NeuronCriteria:
 
         return indent(combined, prefix)[len(prefix):]
 
-
     @classmethod
     def combined_global_with(cls, neuron_conditions, vars=[], prefix=0):
         if isinstance(prefix, int):
@@ -802,9 +823,7 @@ def where_expr(field, values, regex=False, matchvar='n', valuevar=None):
         f"Please pass a list or a variable name, not {values}"
 
     assert valuevar is None or isinstance(valuevar, str)
-
-    assert not regex or len(values) <= 1, \
-        f"Can't use regex mode with more than one value: {values}"
+    assert not regex or not valuevar, "valuevar is not allowed if using a regex"
 
     if len(values) == 0:
         return ""
@@ -825,18 +844,34 @@ def where_expr(field, values, regex=False, matchvar='n', valuevar=None):
     if None not in values:
         if valuevar:
             return f"{matchvar}.{field} in {valuevar}"
+        elif regex:
+            assert all(isinstance(v, str) for v in values), \
+                "Expected all regex values to be strings"
+            r = '|'.join(f'({v})' for v in values)
+            return f"{matchvar}.{field} =~ '{r}'"
         else:
             return f"{matchvar}.{field} in {[*values]}"
 
     # ['some_val', None, 'some_other']
     values = [*filter(lambda v: v is not None, values)]
     if len(values) == 1:
-        if isinstance(values[0], str):
+        if regex:
+            assert isinstance(values[0], str), \
+                "Expected all regex values to be strings"
+            return f"{matchvar}.{field} =~ '{values[0]}' OR NOT exists({matchvar}.{field})"
+        elif isinstance(values[0], str):
             return f"{matchvar}.{field} = '{values[0]}' OR NOT exists({matchvar}.{field})"
         else:
             return f"{matchvar}.{field} = {values[0]} OR NOT exists({matchvar}.{field})"
     else:
-        if valuevar:
+        if regex:
+            # Combine the list fo regexes into a single regex
+            # of the form: '(regex1)|(regex2)|(regex3)'
+            assert all(isinstance(v, str) for v in values), \
+                "Expected all regex values to be strings"
+            r = '|'.join(f'({v})' for v in values)
+            return f"{matchvar}.{field} =~ '{r}' OR NOT exists({matchvar}.{field})"
+        elif valuevar:
             return f"{matchvar}.{field} in {valuevar} OR NOT exists({matchvar}.{field})"
         else:
             return f"{matchvar}.{field} in {[*values]} OR NOT exists({matchvar}.{field})"
